clear;
%% 1. Create Synthetic Data.
TrueModel.T = 250;              %Number of synthetic documents to generate.
TrueModel.GENERATE_HMM = true;  %Synthetic documents correlated?
COND=444;                       %Choose the synthetic pattern.
TrueModel.eps = 0.01;           %Amount of noise in the data.
TrueModel = setup_params(COND,TrueModel);   %Select synthetic dataset params.
TrueModel.Nz = size(TrueModel.psi,1); 
[TrueModel.Nx,TrueModel.Ny] = size(TrueModel.phi);
SIM.Nx = TrueModel.Nx;          %Vocabulary size.
SIM.Nd = TrueModel.Nd;          %Number of words per document.

% Synthetic data: 
% SIM.X: Observed words.
% SIM.Y: True topics
% SIM.Z: True clusters.
[SIM.X, SIM.Y, SIM.Z] = util_make_synthetic(TrueModel); 
figure(1); clf; util_showData(SIM.X,SIM.Nx);
disp('1. Data consists of noisy instances from four classes: Vertical, Horizontal, Top-Left and Bottom-Right'); 
disp('   Note: Top-Right and Bottom-Left do not exist');

%% 2. Setup learning.
SIM.RESET_RAND = 1;         %Reset random number generation.
SIM.DETAILS = 1;            %Verbosity.
SIM.WAIT_CONVERGE = false;  
SIM.INIT_TRUE = false;      %Initialize true?
SIM.REGULAR_LDA = false;    %Learn regular LDA?
SIM.Nm = 4;                 %Number of independent markov chains to simulate.
SIM.Ns = 300;               %Total number of gibbs sweeps per chain.
SIM.LAG = 100;              %Gibbs sweeps between samples.
SIM.BURN = 200;             %Number of burn-in gibbs sweeps.
SIM.T = TrueModel.T;        %Number of documents to simulate.

SIM.alpha = 5; SIM.gamma = 1; SIM.beta = 1/5; %Hyperparameters.
%SIM.alpha = 1; SIM.gamma = 1; SIM.beta = 1; %Hyperparameters.

SIM.USE_SSL=false;          %No semisupervised learning.
RR = cell(1,4);
IL = cell(1,4);
    
%% Learn simple mixture. 
%     SIM.INFER_HMM = false;
%     SIM.REGULAR_HMM = false;
%     [RR{1},IL{1}]=gibbs_MC_LDA8(SIM, [4, 4]);

%% Learn HMM.
%     SIM.INFER_HMM = true;
%     SIM.REGULAR_HMM = true;    
%      SIM.program_name = 'demo_CompModels1 - HMM';
%     [RR{2},IL{2}]=gibbs_MC_LDA8(SIM, [4, 4]);
    
%% Learn MCTM.
% Note can do "matlabpool open 4" to open four workers and parallelize the Markov chains.
    SIM.INFER_HMM = true;
    SIM.REGULAR_HMM = false;
    SIM.program_name = 'demo_Synthetic1 - MCLDA';
    [RR{3},IL{3}]=gibbs_MC_LDA8(SIM, [4, 4]);
    
% %% Learn regular LDA    
%    SIM.REGULAR_LDA  = true;
%    SIM.INFER_HMM    = false;
%    SIM.REGULAR_HMM  = false;
%    [RR{4},IL{4}]=gibbs_MC_LDA8(SIM, [4, 4]);
    
%% Results.
%load('demoSynthetic1.mat'); %Load pre-computed data.
figure(2); clf; util_showImgPHI(RR{3}.phi_est_s, 3); set(gcf,'name','MCTM Learned Topics/Actions'); 
disp('2. Learned topics/actions should be four lines in the cardinal directions');
figure(3); clf; util_showImgClust(RR{3}.phi_est_s, RR{3}.theta_est_s, 3); set(gcf,'Name','MCTM Learned clusters/behaviors');
disp('3. Learned clusters/behaviors should compose topics from Fig 2 into: Vertical, Horizontal, Top-Left and Bottom-Right');
figure(4); clf; plot(RR{3}.lpD_samp); title('Sample Log-likelihood'); xlabel('Iteration'); ylabel('LLh'); legend('Chain 1','Chain 2','Chain 3','Chain 4');
disp('4. Sample log-likelihood during learning. Any non-converged cluster/topic chains should have lower likelihood here');
save('demoSynthetic1.mat');



