function [theta_est, phi_est, psi_est, chi_est] = util_est_Params(BM,TM)
%% function [theta_est, phi_est, psi_est, chi_est] = util_est_Params(BM,TM)
% Point estimate the theta, phi, psi, chi parameters from the model as their mean given the sample counts.

[~,Nz]=size(BM.n_yz);
[Nx,Ny] = size(BM.n_xy);
T = TM.T;
phi_est = zeros(Nx,Ny);
chi_est = zeros(Ny,T);
psi_est = zeros(1,Nz);
theta_est = zeros(Ny,Nz);
for y = 1 : Ny
    phi_est(:,y)  = (BM.n_xy(:,y) + TM.beta) ./(sum(BM.n_xy(:,y)) + Nx*TM.beta);
end

if(TM.REGULAR_LDA)
    for t = 1 :T
        chi_est(:,t) = (BM.n_ym(:,t) + TM.alpha)./(sum(BM.n_ym(:,t))+Ny*TM.alpha);
    end
end
if(TM.REGULAR_HMM)
    psi_est = zeros(Ny,Ny);      
    for y = 1:Ny
        psi_est(:,y)    = (BM.n_yy(:,y) + TM.gamma)./(sum(BM.n_yy(:,y)) + Ny*TM.gamma);
    end
end
if(~TM.REGULAR_HMM && ~TM.REGULAR_LDA) %Then MCLDA, or MIXLDA.
    if(TM.INFER_HMM)
        psi_est = zeros(Nz,Nz);        
        for z = 1:Nz
            psi_est(:,z)    = (BM.n_zz(:,z) + TM.gamma)./(sum(BM.n_zz(:,z)) + Nz*TM.gamma);
        end
    else
        psi_est = (BM.n_z + TM.gamma)./(sum(BM.n_z) + Nz*TM.gamma);
    end

    for z = 1 : Nz
        theta_est(:,z) =  (BM.n_yz(:,z) + TM.alpha) ./(sum(BM.n_yz(:,z)) + Ny*TM.alpha);
    end
end
