function [lpX_Y, lpY_Z, lpZ, lpXYZ] = util_llh_MCLDA(BM, TM)
%% function llh = util_llh_indep_multi_dir(n_xy, n_sxy, beta)
% Compute sample likelihood p(XYZ), integrating over parameters.
% p(X|Y) is simply the product of all the observation likelihoods -
% integrated over the posterior observation Dirichlet.

T = TM.T;
alpha = TM.alpha;
beta = TM.beta;
gamma = TM.gamma;

n_xy = BM.n_xy;
n_yz = BM.n_yz;
n_zz = BM.n_zz;
n_yy = BM.n_yy;
n_syy = BM.n_syy;
n_sxy = BM.n_sxy;
n_syz = BM.n_syz;
n_szz = BM.n_szz;
n_z = BM.n_z;
n_sz = BM.n_sz;
n_sym = BM.n_sym;
n_ym = BM.n_ym;
%n_m = BM.n_m;
[Nx,Ny] = size(n_xy);
[Nz] = size(n_zz,1);

lpX_Y = Ny * ( gammaln(Nx*beta ) - Nx*gammaln(beta ) ) + ...
        sum ( sum ( gammaln(n_xy + beta ), 1) - gammaln( n_sxy + Nx*beta ) );
if(TM.REGULAR_LDA)
    lpY_Z = T * ( gammaln(Ny*alpha) - Ny*gammaln(alpha) ) + ...
          sum( sum( gammaln(n_ym + alpha), 1) - gammaln( n_sym  + Ny*alpha ) );
    lpZ = 0;
end
if(TM.REGULAR_HMM)
    lpY_Z  = Ny * ( gammaln(Ny*gamma) - Ny*gammaln(gamma) ) + ...
           sum ( sum ( gammaln(n_yy + gamma), 1) - gammaln( n_syy + Ny*gamma ) );
    lpZ  = 0;
end
if(~TM.REGULAR_LDA && ~TM.REGULAR_HMM)
    lpY_Z = Nz * ( gammaln(Ny*alpha) - Ny*gammaln(alpha) ) + ...
            sum ( sum ( gammaln(n_yz + alpha), 1) - gammaln( n_syz + Ny*alpha ) );
    if(TM.INFER_HMM)
        lpZ  = Nz * ( gammaln(Nz*gamma) - Nz*gammaln(gamma) ) + ...
               sum ( sum ( gammaln(n_zz + gamma), 1) - gammaln( n_szz + Nz*gamma ) );
    else
        lpZ   = ( gammaln(Nz*gamma) - Nz*gammaln(gamma) ) + ...
                sum ( gammaln(n_z + gamma)   ) - gammaln( n_sz + Nz*gamma );
    end
end
lpXYZ  = lpX_Y + lpY_Z + lpZ;

