function BM = util_recalculateStats_MCLDA(BM,TM)
%% function BM2 = util_recalculateStats_MCLDA(BM,TM)
% Recalculate sufficient statistics from raw samples.
% MCLDA model.

if(TM.REGULAR_LDA)
    BM = util_recalculateStats_LDA(BM,TM);
    return;
end
if(TM.REGULAR_HMM)
    BM = util_recalculateStats_HMM(BM,TM);
    return;
end

Nx = TM.Nx;
[Ny, Nz]=size(BM.n_yz);

BM.n_xy = zeros(Nx, Ny);  %Latent-obs transition count.
BM.n_yz = zeros(Ny, Nz);  %Latent-obs transition count.
BM.n_zz = zeros(Nz, Nz);  %Cluster-cluster transition count.

BM.n_sxy = zeros(1,Ny);
BM.n_syz = zeros(1,Nz);
BM.n_szz = zeros(1,Nz);

BM.n_z  = hist(BM.sampZ,1:Nz); %Needed for non-HMM case.
BM.n_sz = sum(BM.n_z);

%First step.
z = BM.sampZ(1); t =1;
for i = 1 : TM.Nd
    x = BM.X(i,t); y = BM.sampY(i,t);
    BM.n_xy(x,y) = BM.n_xy(x,y)+1;
    BM.n_yz(y,z) = BM.n_yz(y,z)+1;
end


%Next steps. n_zz only needed for HMM case, but just compute anyway.
for t = 2 : TM.T
    z  = BM.sampZ(t); 
    zo = BM.sampZ(t-1);
    BM.n_zz(z,zo) = BM.n_zz(z,zo)+1;

    %BM.n_yz(:,z) = BM.n_yz(:,z) + hist(BM.sampY(:,t));
    for i = 1 : TM.Nd
        x = BM.X(i,t); y = BM.sampY(i,t);
        BM.n_xy(x,y) = BM.n_xy(x,y)+1;
        BM.n_yz(y,z) = BM.n_yz(y,z)+1;
    end
end
BM.n_szz = sum(BM.n_zz,1);


BM.n_sxy = sum(BM.n_xy,1);
BM.n_syz = sum(BM.n_yz,1);


function BM = util_recalculateStats_HMM(BM,TM)
[Nx, Ny]=size(BM.n_xy);

BM.n_xy = zeros(Nx, Ny);  %Latent-obs transition count.
BM.n_yy = zeros(Ny, Ny);  %Cluster-cluster transition count.

BM.n_sxy = zeros(1,Ny);
BM.n_syy = zeros(1,Ny);

%BM.n_z  = hist(BM.sampZ,1:Nz); %Needed for non-HMM case.
BM.n_sy = sum(BM.n_y);

%First step.
y = BM.sampZ(1); t =1;
for i = 1 : TM.Nd
    x = BM.X(i,t); 
    BM.n_xy(x,y) = BM.n_xy(x,y)+1;
    %BM.n_yz(y,z) = BM.n_yz(y,z)+1;
end

%Next steps. n_zz only needed for HMM case, but just compute anyway.
for t = 2 : TM.T
    y  = BM.sampZ(t); 
    yo = BM.sampZ(t-1);
    BM.n_yy(y,yo) = BM.n_yy(y,yo)+1;

    %BM.n_yz(:,z) = BM.n_yz(:,z) + hist(BM.sampY(:,t));
    for i = 1 : TM.Nd
        x = BM.X(i,t);
        BM.n_xy(x,y) = BM.n_xy(x,y)+1;
    end
end
BM.n_syy = sum(BM.n_yy,1);
BM.n_sxy = sum(BM.n_xy,1);
